import unittest
import json

from ejpiaj.variable_extractor import json_variables_extractor


class Response(object):
    def __init__(self, content):
        self.content = content


class TestJsonVariablesxtractor(unittest.TestCase):

    def test_flat_extract(self):
        variables = {
            'var1': 'target1',
            'var2': 'target2',
        }
        response = Response(json.dumps({
            'var1': 'data1',
            'var2': 'data2',
        }))
        result = json_variables_extractor(response, variables)

        self.assertEquals(result['target1'], 'data1')
        self.assertEquals(result['target2'], 'data2')

    def test_children_extract(self):
        variables = {
            'var1.var11': 'target1',
            'var1.var22': 'target2',
        }
        response = Response(json.dumps({
            'var1': {'var11': 'data11', 'var22': 'data22'},
        }))
        result = json_variables_extractor(response, variables)

        self.assertEquals(result['target1'], 'data11')
        self.assertEquals(result['target2'], 'data22')

    def test_list_extract(self):
        variables = {
            '[0].var1.var22.[2]': 'target1',
            '[1].var5.var52': 'target2',
        }
        response = Response(json.dumps([
            {'var1': {'var11': 'data11', 'var22': [1, 2, 3]}, },
            {'var5': {'var51': 'data51', 'var52': 'data52'}, },
        ]))
        result = json_variables_extractor(response, variables)

        self.assertEquals(result['target1'], 3)
        self.assertEquals(result['target2'], 'data52')

    def test_list_with_children_with_list_extract(self):
        variables = {
            'items.[0].item_name': 'target1',
            'items.[1].item_name': 'target2',
        }
        response = Response(json.dumps(
            {
                'items': [
                    {'item_name': 'name1', 'var22': [1, 2, 3]},
                    {'item_name': 'name2', 'var22': [1, 2, 3]},
                ],
                'total_count': 2
            },
        ))
        result = json_variables_extractor(response, variables)

        self.assertEquals(result['target1'], 'name1')
        self.assertEquals(result['target2'], 'name2')



# -*- coding: utf-8 -*-

import logging

import requests
from clint.textui import puts, colored

from .registry import get_variables_extractor, get_assertion


def console_runner(suite, variables=None, debug=False):
    logger = logging.getLogger(__name__)
    failed_counter = 0

    if not variables:
        variables = {}

    puts(colored.white('-' * 80))
    puts(colored.white('P - passed assertions, F - failed assertions'
                       ', V - extracted variables'))
    puts(colored.white('-' * 80))

    tests = suite['requests']
    for test_name in sorted(suite['requests'].keys()):
        test = tests[test_name]
        result = test_request(test, variables)
        logger.debug(result)
        if result['valid']:
            test_name = "%s [P%d,F%d,V%d]" % (
                test_name,
                len(result['passed_assertions']),
                len(result['failed_assertions']),
                len(result['variables']),
            )
            puts(colored.green(
                "✓ " + test_name + " " + str(result['variables'])
            ))
        else:
            failed_counter += 1
            puts(colored.red(
                "✗ " + test_name +
                " (" +
                " ∥ ".join(result['failed_assertions']) + ")"
            ))

    puts(colored.white('-' * 80))

    return failed_counter


def test_request(request, variables):
    method = getattr(requests, request['method'].lower())
    url = _vars(request['url'], variables)
    url_params = None

    failed_assertions = []
    passed_assertions = []
    local_variables = {}
    valid = True

    # headers with applied variables
    headers = _vars(request.get('headers', None), variables)

    # url params with applied variables
    url_params = _vars(request.get('url_params', None), variables)

    # form params with applied variables
    form_params = _vars(request.get('form_params', None), variables)

    # create body data for POST and PUT
    body = None
    if request['method'].lower() in ['post', 'put']:
        if form_params:
            body = form_params
        else:
            body = request.get('body', None)

    response = method(url, params=url_params, data=body, headers=headers)

    # extract variables from response using registered extractors
    if request.get('variables', None):
        for extractor_name, params in request.get('variables').items():
            extractor = get_variables_extractor(extractor_name)
            extractor_variables = extractor(response, params)
            variables.update(extractor_variables)
            local_variables.update(extractor_variables)

    # test assertions using registered extractors
    if request.get('assertions', None):
        for extractor_name, assertions in request.get('assertions').items():
            extractor = get_variables_extractor(extractor_name)
            for assertion in assertions:
                assertion = _vars(assertion, variables)
                if assertion.count(' '):
                    variable, term = assertion.split(' ', 1)
                else:
                    variable = assertion
                value = extractor(response, {variable: 'result'})
                assertion_result = check_assertion(term, value['result'])
                if assertion_result:
                    passed_assertions.append(assertion)
                else:
                    failed_assertions.append(assertion)
                    valid = False

    return {
        'passed_assertions': passed_assertions,
        'failed_assertions': failed_assertions,
        'variables': local_variables,
        'valid': valid,
        'response': response,
        'content': response.content,
    }


def _vars(obj, variables):
    """Set variables in parameters values or body data at .

    Use {{variable_name}} without spaces f.i. in test.yml:

    request_1:
      variables:
        json:
          key: as_name

    request_2:
      url_params
        param: {{as_name}}
    """

    if not obj:
        return

    if isinstance(obj, dict):
        for key, value in obj.items():
            for var_name, var_value in variables.items():
                if var_value is not None:
                    value = value.replace('{{%s}}' % var_name, var_value)
                    obj[key] = value

    if isinstance(obj, str):
        for name, value in variables.items():
            if value is not None:
                obj = obj.replace('{{%s}}' % name, str(value))

    return obj


def check_assertion(expression, value):
    if expression.count(' '):
        assertion_keyword, params = expression.split(' ', 1)
        return get_assertion(assertion_keyword)(value, params)
    else:
        assertion_keyword = expression
        return get_assertion(assertion_keyword)(value)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
test_ejpiaj
----------------------------------

Tests for `ejpiaj` module.
"""

import unittest

from ejpiaj import registry


def extractor1():
    pass


def extractor2():
    pass


class TestEjpiajVariablesRegistry(unittest.TestCase):

    def test_default_registry(self):
        registry.register_variables_extractor('ex1', extractor1)
        registry.register_variables_extractor('ex2', extractor2)
        self.assertEquals(
            registry.get_variables_extractor('ex1'),
            extractor1
        )
        self.assertEquals(
            registry.get_variables_extractor('ex2'),
            extractor2
        )


if __name__ == '__main__':
    unittest.main()



import unittest

from ejpiaj.runner import check_assertion


class Response(object):
    def __init__(self, content):
        self.content = content


class TestAssertions(unittest.TestCase):

    def test_empty_assertion(self):
        self.assertFalse(check_assertion('empty', 'AA'))
        self.assertFalse(check_assertion('empty', '12'))
        self.assertFalse(check_assertion('empty', '1'))
        self.assertFalse(check_assertion('empty', '0'))
        self.assertFalse(check_assertion('empty', 0))
        self.assertTrue(check_assertion('empty', ''))
        self.assertTrue(check_assertion('empty', None))

    def test_notempty_assertion(self):
        self.assertTrue(check_assertion('notempty', 'AA'))
        self.assertTrue(check_assertion('notempty', '12'))
        self.assertTrue(check_assertion('notempty', '1'))
        self.assertTrue(check_assertion('notempty', '0'))
        self.assertTrue(check_assertion('notempty', 0))
        self.assertFalse(check_assertion('notempty', ''))
        self.assertFalse(check_assertion('notempty', None))

    def test_in_assertion(self):
        self.assertTrue(check_assertion('in 1,2,3,4', 1))
        self.assertTrue(check_assertion('in 1,2,3,4', 2))
        self.assertTrue(check_assertion('in 1,2,3,4', 3))
        self.assertTrue(check_assertion('in 1,2,3,4', 4))
        self.assertTrue(check_assertion('in 1,2,3,4', '4'))
        self.assertFalse(check_assertion('in 1,2,3,4', 6))
        self.assertFalse(check_assertion('in 1,2,3,4', '6'))

    def test_notin_assertion(self):
        self.assertFalse(check_assertion('notin 1,2,3,4', 1))
        self.assertFalse(check_assertion('notin 1,2,3,4', 2))
        self.assertFalse(check_assertion('notin 1,2,3,4', 3))
        self.assertFalse(check_assertion('notin 1,2,3,4', 4))
        self.assertFalse(check_assertion('notin 1,2,3,4', '4'))
        self.assertTrue(check_assertion('notin 1,2,3,4', 6))
        self.assertTrue(check_assertion('notin 1,2,3,4', '6'))



import re

import json

from .decorators import variable_extractor


@variable_extractor('json')
def json_variables_extractor(response, variables):
    """Extracts variables from json response.content.

    Variables path are written using 'dot' access and index access to lists
    f.i.:
        some.path.to.list.[0]
        [1].dict.access.later
    """
    result = {}
    re_list = re.compile('^\[\d+\]$')

    # use 'dot' access to dictionary
    data = json.loads(response.content)
    for path, name in variables.items():
        try:
            subdata = data
            for attr in path.split('.'):
                # support for list access [0]
                if re_list.match(attr):
                    ind = int(attr[1:-1])
                    subdata = subdata[ind]
                else:
                    subdata = subdata.get(attr)
            result[name] = subdata
        except:
            result[name] = None
    return result


@variable_extractor('response')
def response_variable_extractor(response, variables):
    result = {}
    re_list = re.compile('^\[\d+\]$')
    for path, name in variables.items():
        try:
            subdata = response
            for attr in path.split('.'):
                # support for list access [0]
                if re_list.match(attr):
                    ind = int(attr[1:-1])
                    subdata = subdata[ind]
                else:
                    subdata = getattr(subdata, attr)
            result[name] = subdata
        except:
            result[name] = None
    return result
